/*
 * glChess - A 3D chess interface
 *
 * Copyright (C) 2002  Robert  Ancell <bob27@users.sourceforge.net>
 *                     Michael Duelli <duelli@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "global.h"
#include "config.h"
#include "player.h"

/* Get one string from a file */
char *get_file_word(FILE * file)
{
  char *word = NULL;
  int i = 1;
  int input_char, is_word;

  do
  {
    /* Clear whitespace */
    do
      input_char = fgetc(file);
    while (isspace(input_char));
    is_word = 1;

    /* Remove comments */
    if (input_char == '#')
    {
      while (input_char != EOF && input_char != '\n')
	input_char = fgetc(file);
      is_word = 0;
    }
  }
  while (!is_word);		/* Repeat until whitespace has been cleared
				 * and no comment is present */

  if (input_char == EOF)
    return NULL;

  /* Malloc 1 char */
  word = malloc(i * sizeof(char));

  /* Get strings (eg "Eat me") */
  if (input_char == '\"')
  {
    while (input_char != '\"' && input_char != EOF)
    {
      word[i - 1] = input_char;
      /* Malloc the next char */
      word = realloc(word, ++i * sizeof(char));
      input_char = fgetc(file);
    }
  } else
  {
    /* Get a word surrounded by whitespace */
    while (!isspace(input_char) && input_char != EOF)
    {
      word[i - 1] = input_char;
      /* Malloc the next char */
      word = realloc(word, ++i * sizeof(char));
      input_char = fgetc(file);
    }
  }
  word[i - 1] = '\0';

  return word;
}

/* Load a rc (config) file */
int open_rc(Game * game, const char *file_name)
{
  FILE *rc;
  char *option = NULL, *value = NULL;

  rc = fopen(file_name, "r");
  if (rc == NULL)
  {
    fprintf(stderr, "%s not found trying /etc/glchessrc\n", file_name);
    rc = fopen("/etc/glchessrc", "r");
    if (rc == NULL)
    {
      fprintf(stderr, "/etc/glchessrc not found, trying ./glchessrc\n");
      rc = fopen("./glchessrc", "r");
    }
  }

  if (rc == NULL)
  {
    fprintf(stderr, "Error opening rc config file...\n");
    return -1;
  }

  while (1)
  {
    /* If no more options */
    if ((option = get_file_word(rc)) == NULL)
      return 1;

    if ((value = get_file_word(rc)) == NULL)
    {
      free(option);
      fprintf(stderr,
	      "Final option found without a corresponding value in %s\n",
	      file_name);
    }

    if (set_variable(game, option, value, rc) == -1)
      fprintf(stderr, "Unknown option in %s.\n", file_name);
  }
  fclose(rc);
}

/* Saves a rc (config) file */
int save_rc(Game * game, const char *fname)
{
  FILE *rc = fopen(fname, "w");
  int i;

  if (rc == NULL)
  {
    fprintf(stderr, "Error opening rc config file (%s)...\n", fname);
    return -1;
  }

  fprintf(rc,
	  "#This is the glChess configuration file, automatically generated by glChess\n");
  fprintf(rc, "%-24s%d\n", "is_reflect", game->is_reflect);
  fprintf(rc, "%-24s%d\n", "is_light", game->is_light);
  fprintf(rc, "%-24s%d\n", "is_smooth", game->is_smooth);
  fprintf(rc, "%-24s%d\n", "is_texture", game->is_texture);
  fprintf(rc, "%-24s%d\n", "is_fog", game->is_fog);
  fprintf(rc, "%-24s%d\n\n", "camera_mode", game->camera->mode);

  fprintf(rc, "%-24s%s\n", "texture_dir", game->texture_dir);
  fprintf(rc, "%-24s", "number_colour");
  for (i = 0; i < 4; i++)
    fprintf(rc, " %f", game->number_colour[i]);
  fprintf(rc, "\n");
  fprintf(rc, "%-23s", "letter_colour");
  for (i = 0; i < 4; i++)
    fprintf(rc, " %f", game->letter_colour[i]);
  fprintf(rc, "\n\n");

  fprintf(rc, "%-24s\"%s\"\n", "white_name", game->white_player->name);
  fprintf(rc, "%-23s", "white_piece_colour");
  for (i = 0; i < 4; i++)
    fprintf(rc, " %f", game->white_player->piece_colour[i]);
  fprintf(rc, "\n");
  fprintf(rc, "%-23s", "white_selected_colour");
  for (i = 0; i < 4; i++)
    fprintf(rc, " %f", game->white_player->selected_colour[i]);
  fprintf(rc, "\n");
  fprintf(rc, "%-23s", "white_board_colour");
  for (i = 0; i < 4; i++)
    fprintf(rc, " %f", game->white_player->board_colour[i]);
  fprintf(rc, "\n");
  fprintf(rc, "%-24s%s\n", "white_texture_dir",
	  game->white_player->texture_dir);
  fprintf(rc, "%-24s%s\n", "white_piece_texture",
	  game->white_player->piece_texture_file);
  fprintf(rc, "%-24s%s\n", "white_selected_texture",
	  game->white_player->selected_piece_texture_file);
  fprintf(rc, "%-24s%s\n\n", "white_board_texture",
	  game->white_player->board_texture_file);

  fprintf(rc, "%-24s\"%s\"\n", "black_name", game->black_player->name);
  fprintf(rc, "%-23s", "black_piece_colour");
  for (i = 0; i < 4; i++)
    fprintf(rc, " %f", game->black_player->piece_colour[i]);
  fprintf(rc, "\n");
  fprintf(rc, "%-23s", "black_selected_colour");
  for (i = 0; i < 4; i++)
    fprintf(rc, " %f", game->black_player->selected_colour[i]);
  fprintf(rc, "\n");
  fprintf(rc, "%-23s", "black_board_colour");
  for (i = 0; i < 4; i++)
    fprintf(rc, " %f", game->black_player->board_colour[i]);
  fprintf(rc, "\n");
  fprintf(rc, "%-24s%s\n", "black_texture_dir",
	  game->black_player->texture_dir);
  fprintf(rc, "%-24s%s\n", "black_piece_texture",
	  game->black_player->piece_texture_file);
  fprintf(rc, "%-24s%s\n", "black_selected_texture",
	  game->black_player->selected_piece_texture_file);
  fprintf(rc, "%-24s%s\n\n", "black_board_texture",
	  game->black_player->board_texture_file);

  fprintf(rc, "%-24s%s\n", "cecp_mode",
	  (game->cecp.mode == CECP_MODE_HARD) ? "HARD" : "EASY");
  fprintf(rc, "%-24s%d\n", "cecp_search_depth", game->cecp.search_depth);
  fprintf(rc, "%-24s%s\n\n", "cecp_ai_player",
	  (game->cecp.ai_player == WHITE PLAYER) ? "WHITE" : "BLACK");

  fprintf(rc, "%-24s%s\n", "path_for_crafty", game->path_crafty);
  fprintf(rc, "%-24s%s\n\n", "path_for_gnuchessx", game->path_gnuchessx);

  fprintf(rc, "%-24s%d\n", "debug", game->cecp.debug);
  fprintf(rc, "%-24s%d\n", "beep_signal", game->cecp.beep_signal);

  fclose(rc);

  return TRUE;
}

/* Gets four floats from the input file */
GLfloat *get_colour(FILE * file, char *var_value)
{
  GLfloat *colour = malloc(sizeof(GLfloat) * 4);

  colour[0] = (GLfloat) atof(var_value);
  colour[1] = (GLfloat) atof(get_file_word(file));
  colour[2] = (GLfloat) atof(get_file_word(file));
  colour[3] = (GLfloat) atof(get_file_word(file));

  return colour;
}

/* Set the variable named by a string */
int set_variable(Game * game, char *var_name, char *var_value, FILE * file)
{
  const char vars[][32] =
      { "is_light", "is_smooth", "is_reflect", "is_texture", "is_fog",
    "camera_mode",
    "white_name",
    "white_piece_colour", "white_selected_colour",
    "white_board_colour",
    "white_texture_dir", "white_piece_texture",
    "white_selected_texture", "white_board_texture",
    "black_name",
    "black_piece_colour", "black_selected_colour",
    "black_board_colour",
    "black_texture_dir", "black_piece_texture",
    "black_selected_texture", "black_board_texture",
    "texture_dir", "number_colour", "letter_colour",
    "cecp_mode", "cecp_search_depth", "cecp_ai_player",
    "path_for_crafty", "path_for_gnuchessx",
    "debug", "beep_signal"
  };
  int i, n_vars = 32;

  for (i = 0; i < n_vars; i++)
    if (!strcmp(var_name, vars[i]))
    {
      switch (i)
      {
      case 0:			/* is_light */
	game->is_light = atoi(var_value);
	break;
      case 1:			/* is_smooth */
	game->is_smooth = atoi(var_value);
	break;
      case 2:			/* is_reflect */
	game->is_reflect = atoi(var_value);
	break;
      case 3:			/* is_texture */
	game->is_texture = atoi(var_value);
	break;
      case 4:			/* is_fog */
	game->is_fog = atoi(var_value);
      case 5:			/* camera_mode */
	game->camera->mode = atoi(var_value);
	break;
      case 6:			/* white_name */
	set_player_name(game->white_player, var_value);
	break;
      case 7:			/* white_piece_colour */
	free(game->white_player->piece_colour);
	game->white_player->piece_colour = get_colour(file, var_value);
	break;
      case 8:			/* white_selected_colour */
	free(game->white_player->selected_colour);
	game->white_player->selected_colour = get_colour(file, var_value);
	break;
      case 9:			/* white_board_colour */
	free(game->white_player->board_colour);
	game->white_player->board_colour = get_colour(file, var_value);
	break;
      case 10:			/* white_texture_dir */
	free(game->white_player->texture_dir);
	game->white_player->texture_dir = strdup(var_value);
	break;
      case 11:			/* white_piece_texture */
	free(game->white_player->piece_texture_file);
	game->white_player->piece_texture_file = strdup(var_value);
	break;
      case 12:			/* white_selected_texture */
	free(game->white_player->selected_piece_texture_file);
	game->white_player->selected_piece_texture_file =
	    strdup(var_value);
	break;
      case 13:			/* white_board_texture */
	free(game->white_player->board_texture_file);
	game->white_player->board_texture_file = strdup(var_value);
	break;
      case 14:			/* black_name */
	set_player_name(game->black_player, var_value);
	break;
      case 15:			/* black_piece_colour */
	free(game->black_player->piece_colour);
	game->black_player->piece_colour = get_colour(file, var_value);
	break;
      case 16:			/* black_selected_colour */
	free(game->black_player->selected_colour);
	game->black_player->selected_colour = get_colour(file, var_value);
	break;
      case 17:			/* black_board_colour */
	free(game->black_player->board_colour);
	game->black_player->board_colour = get_colour(file, var_value);
	break;
      case 18:			/* black_texture_dir */
	free(game->black_player->texture_dir);
	game->black_player->texture_dir = strdup(var_value);
	break;
      case 19:			/* black_piece_texture */
	free(game->black_player->piece_texture_file);
	game->black_player->piece_texture_file = strdup(var_value);
	break;
      case 20:			/* black_selected_texture */
	free(game->black_player->selected_piece_texture_file);
	game->black_player->selected_piece_texture_file =
	    strdup(var_value);
	break;
      case 21:			/* black_board_texture */
	free(game->black_player->board_texture_file);
	game->black_player->board_texture_file = strdup(var_value);
	break;
      case 22:			/* texture_dir */
	free(game->texture_dir);
	game->texture_dir = strdup(var_value);
	break;
      case 23:			/* number_colour */
	free(game->number_colour);
	game->number_colour = get_colour(file, var_value);
	break;
      case 24:			/* letter_colour */
	free(game->letter_colour);
	game->letter_colour = get_colour(file, var_value);
	break;
      case 25:			/* cecp_mode */
	if (!strcmp(var_value, "EASY"))
	  game->cecp.mode = CECP_MODE_EASY;
	else if (!strcmp(var_value, "HARD"))
	  game->cecp.mode = CECP_MODE_HARD;
	else
	{
	  fprintf(stderr, "Warning: Unknown CECP mode: %s\n", var_value);
	  game->cecp.mode = CECP_MODE_UNKNOWN;
	}
	break;
      case 26:			/* cecp_search_depth */
	game->cecp.search_depth = atoi(var_value);
	break;
      case 27:			/* cecp_ai_player */
	if (!strcmp(var_value, "WHITE"))
	  game->cecp.ai_player = WHITE PLAYER;
	else if (!strcmp(var_value, "BLACK"))
	  game->cecp.ai_player = BLACK PLAYER;
	else
	{
	  fprintf(stderr, "Warning: Unknown CECP player: %s\n", var_value);
	  game->cecp.ai_player = BLACK PLAYER;
	}
	break;
      case 28:			/* binary path for crafty */
	game->path_crafty = strdup(var_value);
	break;
      case 29:			/* binary path for gnuchessx */
	game->path_gnuchessx = strdup(var_value);
	break;
      case 30:			/* debug */
	game->cecp.debug = atoi(var_value);
	break;
      case 31:			/* beep_signal */
	game->cecp.beep_signal = atoi(var_value);
	break;
      }
    }

  free(var_name);
  free(var_value);

  return 1;
}
