/*
 * glChess - A 3D chess interface
 *
 * Copyright (C) 2002  Robert  Ancell <bob27@users.sourceforge.net>
 *                     Michael Duelli <duelli@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>
#include <sys/poll.h>
#include <sys/types.h>
#include <signal.h>

#include "global.h"
#include "san.h"
#include "engine.h"
#include "dialog.h"
#include "game.h"

extern int engine;
extern int promotion;

#define MAXBUF 256
#define TIME_OUT 150		/* time out in milliseconds */

int pid;
int input_pipe[2];
int output_pipe[2] = { -1, -1 };	/* -1 means disabled */

/* Simple short abbreviations everybody can remember */
#define INPUT   input_pipe[0]
#define OUTPUT output_pipe[1]

/* Prepares the pipes, starts the engine */
void init_communication(char *bin, char **opt, Game * game)
{
  /* Create the pipes */
  if (pipe(input_pipe) || pipe(output_pipe))
  {
    if (game->cecp.debug)
      debug_output
	  ("DEBUG ( %11s line %3d ): An error occured while creating the pipes !\n",
	   __FILE__, __LINE__);
    exit(-1);
  }

  signal(SIGPIPE, SIG_IGN);

  pid = fork();
  if (!pid)
  {
    dup2(input_pipe[1], STDOUT_FILENO);
    dup2(output_pipe[0], STDIN_FILENO);

    close(input_pipe[0]);
    close(output_pipe[1]);

    setpgid(getpid(), 0);
    execvp(bin, opt);
    /* This won't be reached until the engine won't start for some reasons */
    if (game->cecp.debug)
      debug_output
	  ("DEBUG ( %11s line %3d ): An error occured while starting the engine !\n",
	   __FILE__, __LINE__);
    _exit(1);
  } else
  {
    if (engine == CRAFTY)
      clear_startup_msg(game);
    if (engine == GNUCHESSX)
      clear_engine_msg(game);
  }
}

void clear_startup_msg(Game * game)
{
  char msg[MAXBUF];

  /* Initialize message buffer */
  msg[0] = '\0';
  
  while (strcmp(msg, "White(1)") != 0)
  {
    char *s = msg;

    char c;

    while (read_char(&c) > 0)
    {
      *s = c;
      if (*s == '\n' || *s == ':')
	break;
      ++s;
    }
    *s = '\0';

    if (game->cecp.debug)
      debug_output("DEBUG ( %11s line %3d ): READ_STARTUP_MSG: %s\n",
		   __FILE__, __LINE__, msg);
  }
}

/* Read a char without reading for ever */
int read_char(char *ch)
{
  int ret;
  struct pollfd pfd;

  pfd.fd = INPUT;
  pfd.events = POLLIN | POLLERR | POLLHUP;

  /* If time runs out and there's still no char, we guess there won't be
   * any more */
  if (poll(&pfd, 1, TIME_OUT) < 0)
  {
    return (-1);
  }

  if ((pfd.revents & (POLLERR | POLLHUP)) != 0)
    return (-1);

  if ((pfd.revents & POLLIN) != 0)
    ret = read(INPUT, ch, 1);
  else
    ret = 0;

  return ret;
}

/* Inits the CEC Protocol interface */
void init_cecp_interface(Game * game)
{
  char *bin = NULL;
  char *args[2];
  char sd[10];

  switch (engine)
  {
  case CRAFTY:
    bin = game->path_crafty;
    break;
  case GNUCHESSX:
    bin = game->path_gnuchessx;
    break;
  default:			/* Otherwise play local Human vs. local Human */
    return;
    break;
  }

  args[0] = bin;
  args[1] = NULL;

  init_communication(bin, args, game);

  if (engine == CRAFTY)
  {
    /* We need this for CECP init ! */
    write_to_engine("xboard", game);
    clear_engine_msg(game);
  } else if (engine == GNUCHESSX)
  {
  	write_to_engine("xboard", game);
	clear_engine_msg(game);
  }

  if (engine == CRAFTY)
  {
    /* Set the mode */
    if (game->cecp.mode == CECP_MODE_EASY)
      write_to_engine("easy", game);
    else
      /* FIXME: This is not implemented at all until now */
      write_to_engine("hard", game);
    clear_engine_msg(game);
  }

  if (engine == CRAFTY)
  {
    /* Set the search depth */
    snprintf(sd, 10, "sd %d", game->cecp.search_depth);
    write_to_engine(sd, game);
  } else if (engine == GNUCHESSX)
  {
    snprintf(sd, 10, "%d", game->cecp.search_depth);
    write_to_engine("depth", game);
    write_to_engine(sd, game);
  }
  clear_engine_msg(game);
}

/* Tells if the CECP interface is up */
int cecp_interface_up(void)
{
  if (OUTPUT == -1)
    return FALSE;
  else
    return TRUE;
}

/* Closes the CEC Protocol interface */
void close_cecp_interface(Game * game)
{
  /* If already closed */
  if (OUTPUT == -1)
    return;

  /* Shut the engine down */
  write_to_engine("quit", game);
  /* Clear quit messages */
  clear_engine_msg(game);

  if (pid)
    kill(-pid, SIGKILL);
  pid = 0;

  /* Set the interface to be closed */
  OUTPUT = -1;
}

/* Engine might print some useless message, get rid of these */
void clear_engine_msg(Game * game)
{
  int size_of_s = sizeof(char) * MAXBUF;
  int rest_of_s = size_of_s;
  char *s = malloc(size_of_s + 1);		/* Make room for terminating zero char */
  char *buf = s;
  char c;

  /* Avoid buffer overflow and memory corruption */
  while (rest_of_s-- > 0 && read_char(&c) > 0)
    *buf++ = c;
  *buf = '\0';		/* Make sure string is zero terminated */	

  /* Replace first new line char with zero char, removing rest of string */
  if ((buf = strchr(s, '\n')) != NULL)
    *buf = '\0';

  if (game->cecp.debug)
    debug_output("DEBUG ( %11s line %3d ): CLEAR_ENGINE_MSG: \"%s\"\n",
		 __FILE__, __LINE__, s);

  free(s);
}

/* Writes text to the engine */
void write_to_engine(char *str, Game * game)
{
  char *buf = malloc(sizeof(char) * MAXBUF);

  sprintf(buf, "%s\n", str);
  write(OUTPUT, buf, strlen(buf));

  if (game->cecp.debug)
    debug_output("DEBUG ( %11s line %3d ): WRITE_TO_ENGINE: %s\n",
		 __FILE__, __LINE__, buf);

  free(buf);
}

/* Reads text from the engine */
char *read_from_engine(Game * game)
{
  int rest_of_msg;
  char msg[MAXBUF + 1];		/* Make room for terminating zero char */
  char *s = msg;
  char c;

  /* Initialize message buffer */
  msg[0] = '\0';

   while ((engine == CRAFTY && strstr(msg, "move") == NULL)
	 || (engine == GNUCHESSX && strstr(msg, "move is:") == NULL))
  {
    /* Skip previous message */
	s = msg;
	rest_of_msg = MAXBUF;
    while (rest_of_msg-- > 0 && read_char(&c) > 0)
    {
      *s++ = c;

      if (c == '\n')
		break;
    }
	*s = '\0';
  }
  *s = '\0';

  if (game->cecp.debug)
    debug_output("DEBUG ( %11s line %3d ): READ_FROM_ENGINE: %s\n",
		 __FILE__, __LINE__, msg);

  return strdup(msg);
}

/* Convert glChess' internal to coordinated notation */
char *convert_notation(int *target, Game * game)
{
  char *move = malloc(sizeof(char) * 6);	/* e.g. e2e4 or e7e8Q */

  if (promotion != -1)
  {
    int c;

    switch (promotion)
    {
    case QUEEN:
      c = 'Q';
      break;
    case KNIGHT:
      c = 'N';
      break;
    case BISHOP:
      c = 'B';
      break;
    case ROOK:
      c = 'R';
      break;
    default:
      if (game->cecp.debug)
	debug_output
	    ("DEBUG ( %11s line %3d ): Unknown promotion piec %d\n",
	     __FILE__, __LINE__, promotion);
      break;
    }
    snprintf(move, 6, "%c%d%c%d%c",
	     'a' + game->selected[0], 1 + game->selected[1],
	     'a' + target[0], 1 + target[1], c);
  } else
  {
    snprintf(move, 6, "%c%d%c%d",
	     'a' + game->selected[0], 1 + game->selected[1],
	     'a' + target[0], 1 + target[1]);
  }

  if (game->cecp.debug)
    debug_output("DEBUG ( %11s line %3d ): CONVERT_NOTATION: %s\n",
		 __FILE__, __LINE__, move);

  return move;
}

/* Checks whether move is in coordinated or SAN notation */
int check_notation_format(char *move)
{
  if (isalpha(move[0]) && isdigit(move[1]) &&
      isalpha(move[2]) && isdigit(move[3]))
    return 1;			/* Coordinated */

  return 0;			/* SAN */
}

/* Parses the engine's move */
int parse_move_from_engine(Game * game, char *in, int *start, int *target)
{
  char *s = in;
  char *move_prefix;

  if (strstr(in, "learn"))	/* Ignore learning messages (Crafty only) */
  {
    s = strstr(in, "move");

    if (game->cecp.debug)
      debug_output("DEBUG ( %11s line %3d ): PARSE_MOVE_FROM_ENGINE: %s",
		   __FILE__, __LINE__, s);
  } else if (strchr(in, '#') || strstr(in, "mate"))
  {
    engine_dialog("The game is over");

    /* The game is already over */
    if (game->cecp.debug)
      debug_output("DEBUG ( %11s line %3d ): RESULT: %s", __FILE__,
		   __LINE__, in);
  }

  /* Otherwise, let's parse the move */

  if (engine == CRAFTY)
  {
    /* Get rid of the 'move ' prefix */
    move_prefix = "move ";		/* e.g. move e2e4 */
  } else if (engine == GNUCHESSX)
  {
    /* Get rid of the whole shebang GNU Chess creates, e.g. */
	move_prefix = "My move is: ";	/* My move is: g7g6 */
  }
  s = strstr(s, move_prefix) + strlen(move_prefix);

  if (check_notation_format(s) == 0)	/* SAN */
  {
    Game *game_copy;
      
    if (game->cecp.debug)
      debug_output("DEBUG ( %11s line %3d ): CHECK_NOTATION_FORMAT: SAN\n",
		   __FILE__, __LINE__);
      
    game_copy = copy_game(game);

    s = san_to_coordinated(game_copy, s);
    free(game_copy);
  } else
  {
    if (game->cecp.debug)
      debug_output
	  ("DEBUG ( %11s line %3d ): CHECK_NOTATION_FORMAT: Coordinated\n",
	   __FILE__, __LINE__);

    promotion = get_promotion_piece(s);
  }

#if 1
  start[0] = *s++ - 97;
  start[1] = *s++ - 49;
  target[0] = *s++ - 97;
  target[1] = *s - 49;
#else
  /* Equivilant but clearer notation: */
  start[0] = *s++ - 'a';
  start[1] = *s++ - '1';
  target[0] = *s++ - 'a';
  target[1] = *s - '1';
#endif

  if (game->cecp.debug)
    debug_output
	("DEBUG ( %11s line %3d ): PARSE_MOVE_FROM_ENGINE: %d,%d,%d,%d\n",
	 __FILE__, __LINE__, start[0], start[1], target[0], target[1]);

  return 1;			/* Everything alright */
}

/* Makes the speaker beep on illegal move */
void beep(void)
{
  putc(7, stderr);
}

/* Just for simplifying debug output */
void debug_output(char *format, ...)
{
  va_list ap;

  va_start(ap, format);
  vfprintf(stderr, format, ap);
  va_end(ap);
}

/* Extracts the promotion piece */
int get_promotion_piece(char *str)
{
  char *s;

  /* Promotion piece is before newline char */
  if ((s = strchr(str, '\n')) != NULL)
  {
    switch (tolower(*--s))	/* The promotion piece, e.g. e7e8q, e8=Q */
    {
    case 'q':
      return QUEEN;
      break;
    case 'r':
      return ROOK;
      break;
    case 'n':
      return KNIGHT;
      break;
    case 'b':
      return BISHOP;
      break;
    default:
      break;
    }
  }

  return -1;			/* No promotion */
}

/* Searches for a binary, similar to the 'which' tool */
char *binary_path(char *bin)
{
  char *path = NULL;
  char *str = NULL;
  char *fullpath = NULL;

  path = strdup(getenv("PATH"));

  while (*path != '\0')
  {
    str = path;

    while (*path != ':' && *path != '\0')
      path++;

    if (*path != '\0')
    {
      *path = '\0';
      path++;
    }

    fullpath = malloc(strlen(str) + strlen(bin) + 2);
    snprintf(fullpath, strlen(str) + strlen(bin) + 2, "%s/%s", str, bin);

    if (fopen(fullpath, "r") != NULL)
      return fullpath;

    free(fullpath);
  }

  return NULL;
}
