/*
 * glChess - A 3D chess interface
 *
 * Copyright (C) 2002  Robert  Ancell <bob27@users.sourceforge.net>
 *                     Michael Duelli <duelli@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* move.c: Contains the mouse functions, selection algorithm, the move list
 	   functions, the check if 'in check' algorithm and the move rules. */

#include <stdio.h>
#include <stdlib.h>
#include <GL/glu.h>

#include "global.h"
#include "move.h"
#include "draw.h"
#include "engine.h"
#include "game.h"
#include "menu.h"
#include "dialog.h"

extern int engine;
extern int promotion;
extern int *move_list;
extern int move_list_bufsize;

/* Click the screen */
void mouse(glChessWidget * glcwidget, int button, int x, int y)
{
  int square[2];
  int color;
  Player *player;
  Game *game = glcwidget->game;

  game->camera->oldx = x;
  game->camera->oldy = y;

  /* You select twice. The start piece and the target square */
  if (pick_square(game, x, y, square) != -1)
  {
    if (game->selected[0] != -1)	/* second selection: target */
    {
      if (game->selected[0] == square[0]	/* Check if null move */
	  && game->selected[1] == square[1])
      {
	game->selected[0] = -1;	/* unselect piece */
	return;
      }

      if (engine != HUMAN)
      {
	/* Your legality proofed move... */
	if (move_piece(game, square) == -1)
	{
	  /* Reset selection */
	  game->selected[0] = -1;
	  return;
	}
	/* Add move to move list */
	move_list_add_move(glcwidget, game->selected[0], game->selected[1],
			   square[0], square[1], promotion);
	promotion = -1;

	/* Now tell the engine your move... */
	write_to_engine(convert_notation(square, game), game);

	/* Reset selection */
	game->selected[0] = -1;

	/* Now parse the engine's move ... */
	parse_move_from_engine(game,
			       read_from_engine(game),
			       game->selected, square);

	/* ... now execute the engine's move */
	if (move_piece(game, square) == -1)
	{
	  /* Reset selection */
	  game->selected[0] = -1;
	  return;
	}
	/* Add move to move list */
	move_list_add_move(glcwidget, game->selected[0], game->selected[1],
			   square[0], square[1], promotion);
	/* Reset selection */
	game->selected[0] = -1;

      } else			/* If Human */
      {
	if (move_piece(game, square) == -1)
	{
	  /* Reset selection */
	  game->selected[0] = -1;
	  return;
	}
	/* Add move to move list */
	move_list_add_move(glcwidget, game->selected[0], game->selected[1],
			   square[0], square[1], promotion);
	promotion = -1;

	/* Reset selection */
	game->selected[0] = -1;
      }
    } else			/* first selection: start */
    {
      color = game->board[square[0]][square[1]];
      promotion = -1;

      if (color > 10)
	player = game->black_player;
      else
	player = game->white_player;

      if (game->board[square[0]][square[1]] != EMPTY
	  && player == game->current_player)
      {
	game->selected[0] = square[0];
	game->selected[1] = square[1];
      }

      /* Assimilate color for promotion check */
      if (color > 10)
	color = 0;		/* black */
      else
	color = 1;		/* white */

      /* Promotion: A pawn which is one line from opponent's base line
       * is selected, ask the user which promotion piece he prefers */
      if (square[1] == 5 * color + 1 &&
	  game->board[square[0]][square[1]] == 10 * (1 - color) + PAWN)
      {
	promotion_dialog();
      }
    }
  }
}

/* The trackball */
void mouse_move(Game * game, int button, int x, int y)
{
  int dx, dy;

  if (game->camera->mode != MODE_TRACKBALL)
    return;

  dx = x - game->camera->oldx;
  dy = y - game->camera->oldy;

  if (button == 1)
  {
    game->camera->track_rot[0] += (GLfloat) dx;
    /* Why should we look at the bottom ? I want to see the board not
     * the interior of the pieces ! */
    if (game->camera->track_rot[1] + (GLfloat) dy >= 6.0f &&
	game->camera->track_rot[1] + (GLfloat) dy <= 90.0f)
      game->camera->track_rot[1] += (GLfloat) dy;
  } else if (button == 2)
    if (game->camera->track_rad + (GLfloat) dy >= 10.0f &&
	game->camera->track_rad + (GLfloat) dy <= 250.0f)
      game->camera->track_rad += (GLfloat) dy;

  game->camera->oldx = x;
  game->camera->oldy = y;

  post_redisplay(game);
}

/* Draws the board squares for picking */
void draw_squares(void)
{
  GLuint x, y;

  /* draw the floor squares */
  for (x = 0; x < 8; x++)
  {
    glLoadName(x);

    for (y = 0; y < 8; y++)
    {
      glPushName(y);

      /* Draw square */
      glBegin(GL_QUADS);
      glVertex3f(x * 10.0 - 40.0, 0.0, 40.0 - y * 10.0);
      glVertex3f(x * 10.0 - 30.0, 0.0, 40.0 - y * 10.0);
      glVertex3f(x * 10.0 - 30.0, 0.0, 30.0 - y * 10.0);
      glVertex3f(x * 10.0 - 40.0, 0.0, 30.0 - y * 10.0);
      glEnd();

      glPopName();
    }
  }
}

/* Select the piece under the pointer */
int pick_square(Game * game, int x, int y, int sq[])
{
  GLuint selectBuf[5];
  GLint hits;
  GLint viewport[4];

  glGetIntegerv(GL_VIEWPORT, viewport);

  glSelectBuffer(5, selectBuf);
  (void) glRenderMode(GL_SELECT);

  glInitNames();
  glPushName(0);

  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();

  /* create 5x5 pixel picking region near cursor location */
  gluPickMatrix((GLdouble) x, (GLdouble) (viewport[3] - y), 1.0, 1.0,
		viewport);

  redo_perspective(game, viewport[2], viewport[3]);

  draw_squares();
  glPopMatrix();
  glFlush();

  hits = glRenderMode(GL_RENDER);

  if (hits <= 0)
  {
    glMatrixMode(GL_MODELVIEW);
    return -1;
  }

  sq[0] = selectBuf[3];
  sq[1] = selectBuf[4];

  glMatrixMode(GL_MODELVIEW);

  /* Update view */
  post_redisplay(game);

  return FALSE;
}

/* Moves the selected piece to target[] */
int move_piece(Game * game, int *target)
{
  int start_piece, target_piece;
  int start_color, target_color;

  start_piece =			/* selected piece */
      game->board[game->selected[0]][game->selected[1]];

  /* If black */
  if (start_piece > 10)
  {
    start_color = 0;		/* black */
    start_piece -= 10;
  } else
    start_color = 1;		/* white */

  target_piece = game->board[target[0]][target[1]];	/* target piece */

  /* If black */
  if (target_piece > 10)
  {
    target_color = 0;		/* black */
    target_piece -= 10;
  } else if (target_piece != -1)
    target_color = 1;		/* white */
  else
    target_color = -1;

  /* If target is friendly - null move */
  if (target_color == start_color)
    return -1;

  if (check_move(game, game->selected, target, start_piece, start_color,
		 target_piece, target_color, 1) == -1)
  {
    engine_dialog("Illegal\nmove.");

    if (game->cecp.beep_signal)
      beep();

    return -1;
  }

  /* Move piece */
  game->board[target[0]][target[1]] =
      game->board[game->selected[0]][game->selected[1]];
  game->board[game->selected[0]][game->selected[1]] = EMPTY;

  /* Swap turns */
  if (game->current_player == game->white_player)
    game->current_player = game->black_player;
  else
    game->current_player = game->white_player;
  game->turn++;

  return TRUE;
}

/* Check the move */
int
check_move(Game * game, int *start, int *target, int start_piece,
	   int start_color, int target_piece, int target_color,
	   int check_proof)
{
  int dx, dy, lx, ly;
  int i, j, board_copy[8][8];

  dx = lx = target[0] - start[0];
  dy = ly = target[1] - start[1];

  if (lx < 0)
    lx = -lx;

  if (ly < 0)
    ly = -ly;


  /* Copy the board for later use */
  for (i = 0; i < 8; i++)
    for (j = 0; j < 8; j++)
      board_copy[i][j] = game->board[i][j];

  switch (start_piece)
  {
  case KING:
    /* Castling */
    if (lx == 2 && ly == 0 && start[1] == 7 * (1 - start_color)
	&& check_ortho(game, start, target))
    {
      int skipped[2];

      skipped[1] = target[1];
      if (target[0] == 6 &&	/* short castling */
	  game->board[7][target[1]] == ROOK + 10 * (1 - start_color))
      {
	skipped[0] = target[0] - 1;

	/* Check if king or rook already have been moved */
	if (!move_list_check_piece(game, 4, target[1])	/* king */
	    && !move_list_check_piece(game, 7, target[1])	/* rook */
	    && in_check_check(game, start, skipped,	/* skipped field mustn't be attacked */
			      start_color, target_color) == 0)
	{
	  if (game->cecp.debug)
	    debug_output
		("DEBUG ( %11s line %3d ): Short castling move\n",
		 __FILE__, __LINE__);
	  /* Take the rook... */
	  game->board[7][target[1]] = EMPTY;
	  /* ...and drop it here */
	  game->board[5][target[1]] = ROOK + 10 * (1 - start_color);
	} else
	  return -1;
      } else if (target[0] == 2 &&	/* long castling */
		 game->board[0][target[1]] ==
		 ROOK + 10 * (1 - start_color))
      {
	skipped[0] = target[0] + 1;
	if (!move_list_check_piece(game, 4, target[1])	/* king */
	    && !move_list_check_piece(game, 0, target[1])	/* rook */
	    && in_check_check(game, start, skipped,	/* skipped field mustn't be attacked */
			      start_color, target_color) == 0)
	{
	  if (game->cecp.debug)
	    debug_output("DEBUG ( %11s line %3d ): Long castling move\n",
			 __FILE__, __LINE__);
	  /* Take the rook... */
	  game->board[0][target[1]] = EMPTY;
	  /* ...and drop it here */
	  game->board[3][target[1]] = ROOK + 10 * (1 - start_color);
	} else
	  return -1;
      } else
	return -1;
    } else if (lx > 1 || ly > 1)
      return -1;
    break;

  case QUEEN:
    if (dx == 0 || dy == 0)	/* horizontal or vertical */
    {
      if (!check_ortho(game, start, target))
	return -1;
    } else if (lx == ly)	/* diagonal */
    {
      if (!check_diag(game, start, target))
	return -1;
    } else
      return -1;
    break;

  case ROOK:
    if (dx == 0 || dy == 0)
    {
      if (!check_ortho(game, start, target))
	return -1;
    } else
      return -1;
    break;

  case BISHOP:
    if (lx == ly)
    {
      if (!check_diag(game, start, target))
	return -1;
    } else
      return -1;
    break;

  case KNIGHT:
    if (lx * ly != 2)
      return -1;
    break;

  case PAWN:
    /* Check for 'en passant' move */
    if (lx == 1 && ly == 1
	&& target_piece == EMPTY
	&& start[1] == 3 + start_color
	&& game->board[target[0]][start[1]] == PAWN + 10 * start_color)
    {
      /* Check if 'en passant' move is directly after enemy pawn move */
      if (move_list_check_piece(game, target[0], start[1]) == game->turn)
      {
	if (game->cecp.debug)
	  debug_output("DEBUG ( %11s line %3d ): En passant move\n",
		       __FILE__, __LINE__);
	game->board[target[0]][start[1]] = EMPTY;	/* Remove enemy pawn */
	break;
      }
    }

    /* If moving "forwards" 1 unit and laterally no more than 1 unit */
    if (dy == 2 * start_color - 1 && lx <= 1)
    {
      /* Moving diagonally, but not into an enemy */
      if (lx == 1 && target_piece == EMPTY)
	return -1;

      /* If moving forward into enemy */
      if (dx == 0 && target_piece != EMPTY)
	return -1;

      /* Pawn promotion */
      if (start[1] == 5 * start_color + 1 && target[1] == 7 * start_color)
      {
	if (game->cecp.debug)
	  debug_output("DEBUG ( %11s line %3d ): Promotion !!\n",
		       __FILE__, __LINE__);

	game->board[start[0]][start[1]] =
	    promotion + 10 * (1 - start_color);
      }
    } else if (dy == 4 * start_color - 2 && lx == 0)
    {
      if (start[1] != 6 - 5 * start_color
	  || game->board[start[0]][5 - 3 * start_color] != EMPTY
	  || target_piece != EMPTY)
	return -1;
    } else
      return -1;

    break;
  }

  /* Even if the move itself was legal, it could put the king into
   * chess and consequently the move is illegal though */

  if (check_proof)
  {
    /* The 'in check' check algorithm */
    if (in_check_check(game, start, target, start_color, target_color) ==
	-1)
    {
      if (game->cecp.debug)
	debug_output
	    ("DEBUG ( %11s line %3d ): King would be in check !!\n",
	     __FILE__, __LINE__);

      /* Restore old game->board by re-copying */
      if (start_piece == KING || start_piece == PAWN)
	for (i = 0; i < 8; i++)
	  for (j = 0; j < 8; j++)
	    game->board[i][j] = board_copy[i][j];

      return -1;
    }
  }

  return 1;
}

/* Check the (orthogonal) line from source[] to dest[] is free of pieces */
int check_ortho(Game * game, int *source, int *dest)
{
  int step, i;

  if (source[0] == dest[0])
  {				/* along x-axis */
    if (dest[1] > source[1])
      step = 1;
    else
      step = -1;
    for (i = source[1] + step; i != dest[1]; i += step)
    {
      if (game->board[source[0]][i] != EMPTY)
	return FALSE;
    }
  } else
  {				/* along y-axis */
    if (dest[0] > source[0])
      step = 1;
    else
      step = -1;
    for (i = source[0] + step; i != dest[0]; i += step)
    {
      if (game->board[i][source[1]] != EMPTY)
	return FALSE;
    }
  }
  return TRUE;
}

/* Check the diagonal from source[] to dest[] is free of pieces */
int check_diag(Game * game, int *source, int *dest)
{
  int step, i;

  if ((dest[0] - source[0]) == (dest[1] - source[1]))
  {				/* NE/SW */
    if (dest[0] > source[0])
      step = 1;			/* NE */
    else
      step = -1;		/* SW */
    for (i = step; i != dest[0] - source[0]; i += step)
    {
      if (game->board[source[0] + i][source[1] + i] != EMPTY)
	return FALSE;
    }
  } else
  {				/* along NW/SE */

    if (dest[1] > source[1])
      step = 1;			/* NW */
    else
      step = -1;		/* SE */
    for (i = step; i != source[0] - dest[0]; i += step)
    {
      if (game->board[source[0] - i][source[1] + i] != EMPTY)
	return FALSE;
    }
  }
  return TRUE;
}

int in_check_check(Game * game, int *start, int *target, int start_color,
		   int target_color)
{
  Game *game_copy = copy_game(game);

  /* Hypothetically do the move */
  game_copy->board[target[0]][target[1]] =
      game_copy->board[start[0]][start[1]];
  game_copy->board[start[0]][start[1]] = EMPTY;

  /* Hypothetically check if king would be in chess after the move */
  if (king_in_check(game_copy, start_color, target_color))
  {
    free(game_copy);
    return -1;			/* King is in check, do a defensive move */
  }

  free(game_copy);
  return 0;			/* King is not and won't be in check for now */
}

int king_in_check(Game * game, int start_color, int target_color)
{
  int i, j;
  int enemy_pos[2];
  int enemy_piece;
  int king_pos[2];

  /* Get king position */
  for (i = 0; i < 8; i++)
    for (j = 0; j < 8; j++)
      if (game->board[i][j] == KING + 10 * (1 - start_color))
      {
	king_pos[0] = i;
	king_pos[1] = j;
      }

  /* Check if any enemy piece can move to king_pos */
  for (i = 0; i < 8; i++)
    for (j = 0; j < 8; j++)
      if (game->board[i][j] >= 10 * start_color
	  && game->board[i][j] <= 10 * start_color + 6)
      {
	enemy_pos[0] = i;
	enemy_pos[1] = j;

	enemy_piece = game->board[i][j];
	if (enemy_piece > 10)
	{
	  target_color = 0;
	  enemy_piece -= 10;
	} else
	  target_color = 1;

	if (check_move(game, enemy_pos, king_pos, enemy_piece,
		       target_color, KING, start_color, 0) != -1)
	  return 1;
      }

  return 0;			/* King is not in check */
}

/* Create the move list */
int move_list_create(void)
{
  int *line;

  if (move_list != NULL)
  {
    engine_dialog("Move list already exists\n");
    return -1;
  }

  move_list_bufsize += 5;
  move_list = malloc(move_list_bufsize * sizeof(int));

  line = move_list;
  *line++ = -1;			/* Marks the end */

  return 1;
}

/* Free the move list */
int move_list_free(void)
{
  if (move_list != NULL)
  {
    free(move_list);
    move_list = NULL;
    return 1;
  }

  return 0;
}

/* Add a move entry to the move list */
void
move_list_add_move(glChessWidget * glcwidget, int x1, int y1, int x2,
		   int y2, int promotion)
{
  int *line = move_list;
  Game *game = glcwidget->game;
  char text[20];
  int i, j;
  int white = 0;
  int black = 0;
  int c;

  while (*line != -1)
    line++;

  *line++ = x1;
  *line++ = y1;
  *line++ = x2;
  *line++ = y2;
  *line = -1;			/* Marks the end */

  move_list_bufsize += 4;
  move_list = realloc(move_list, move_list_bufsize * sizeof(int));

  if (promotion != -1)
    switch (promotion)
    {
    case QUEEN:
      c = 'Q';
      break;
    case KNIGHT:
      c = 'N';
      break;
    case BISHOP:
      c = 'B';
      break;
    case ROOK:
      c = 'R';
      break;
    default:
      if (game->cecp.debug)
	debug_output
	    ("DEBUG ( %11s line %3d ): Unknown promotion piece %d\n",
	     __FILE__, __LINE__, promotion);
      break;
    }

  /* Now update the clist */
  if (game->turn % 2 == 1)	/* it was White's turn */
  {
    char move_num[4];
    char move[6];

    sprintf(move_num, "%2d.", (game->turn + 1) / 2);

    if (promotion != -1)
      sprintf(move, "%c%d%c%d%c", 'a' + x1, 1 + y1, 'a' + x2, 1 + y2, c);
    else
      sprintf(move, "%c%d%c%d", 'a' + x1, 1 + y1, 'a' + x2, 1 + y2);

    gtk_clist_append(GTK_CLIST(glcwidget->clist), (char *[])
		     {
		     "", "", ""});

    /* Scroll down one line */
    gtk_clist_moveto(GTK_CLIST(glcwidget->clist), (game->turn + 1) / 2 - 1,
		     0, 1.0, 1.0);

    /* Fill the '#' column */
    gtk_clist_set_text(GTK_CLIST(glcwidget->clist),
		       (game->turn + 1) / 2 - 1, 0, move_num);

    /* Fill the 'White' column */
    gtk_clist_set_text(GTK_CLIST(glcwidget->clist),
		       (game->turn + 1) / 2 - 1, 1, move);
  } else			/* it was Black's turn */
  {
    char move[6];

    if (promotion != -1)
      sprintf(move, "%c%d%c%d%c", 'a' + x1, 1 + y1, 'a' + x2, 1 + y2, c);
    else
      sprintf(move, "%c%d%c%d", 'a' + x1, 1 + y1, 'a' + x2, 1 + y2);

    /* Fill the 'Black' column */
    gtk_clist_set_text(GTK_CLIST(glcwidget->clist),
		       (game->turn + 1) / 2 - 1, 2, move);
  }

  /* Calculate material statistic */
  for (i = 0; i < 8; i++)
    for (j = 0; j < 8; j++)
    {
      if (game->board[i][j] < 10)
	switch (game->board[i][j])
	{
	case WHITE PAWN:
	  white += 1;
	  break;
	case WHITE KNIGHT:
	case WHITE BISHOP:
	  white += 3;
	  break;
	case WHITE ROOK:
	  white += 5;
	  break;
	case WHITE QUEEN:
	  white += 9;
	  break;
	default:
	  break;
      } else
	switch (game->board[i][j])
	{
	case BLACK PAWN:
	  black += 1;
	  break;
	case BLACK KNIGHT:
	case BLACK BISHOP:
	  black += 3;
	  break;
	case BLACK ROOK:
	  black += 5;
	  break;
	case BLACK QUEEN:
	  black += 9;
	  break;
	default:
	  break;
	}
    }

  sprintf(text, "Material: %2d / %2d", white, black);

  /* Update material statistic */
  gtk_label_set_text(GTK_LABEL(glcwidget->label), text);
}

/* Check if piece of these coordinates was already moved */
int move_list_check_piece(Game * game, int x, int y)
{
  int move_num;
  int *line = move_list;
  int line_pos;

  for (line_pos = 0; *line != -1; line++, line_pos++)
  {
    move_num = line_pos / 4 + 1;

    if (line_pos % 2 == 0 && *line == x && *(line + 1) == y)
      return move_num;		/* piece was already moved in move num */
  }

  return 0;			/* piece was not moved */
}
