/*
 * glChess - A 3D chess interface
 *
 * Copyright (C) 2002  Robert  Ancell <bobg27@users.sourceforge.net>
 *                     Michael Duelli <duelli@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* san.c: Converts Standard Algebraic Notation (SAN) to coordinated notation */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#include "global.h"
#include "engine.h"
#include "san.h"
#include "move.h"

extern int promotion;

/* If it's a castling move, return 1 or 2 */
int san_check_castling(char *san_str)
{
  if (san_str[1] == '-')
  {
    if (san_str[3] != '-')
      return 1;			/* short */
    else
      return 2;			/* long */
  }

  return 0;			/* No castling move */
}

/* Tell me what kind of piece is moved */
int san_extract_piece(char *san_str)
{
  switch (san_str[0])
  {
  case 'K':
    return KING;
  case 'Q':
    return QUEEN;
  case 'B':
    return BISHOP;
  case 'N':
    return KNIGHT;
  case 'R':
    return ROOK;
  default:
    return PAWN;
  }
}

/* Get rid of the 'x', '+', '=', '#' and the newline char */
void san_simplify(char *san_str)
{
  char *s = san_str;
  char *t = san_str;

  while (*s != '\n')
  {
    if (*s == 'x' || *s == '+' || *s == '#' || *s == '=')
    {
      if (*s == '=')
      {
	if ((promotion = san_extract_piece(++s)) == PAWN)
	  promotion = -1;
      }

      s++;
    } else if (*s == 'K' || *s == 'Q' || *s == 'R' || *s == 'N'
	       || *s == 'B')
      s++;
    else
      *t++ = *s++;
  }
  *t = '\0';
}

/* Convert SAN to coordinated */
char *san_to_coordinated(Game * game, char *san_str)
{
  int start_piece, start_color;
  int target_piece, target_color;
  int castling;
  int start[2];
  int target[2];
  char *move = malloc(sizeof(char) * 5);
  int i, j;

  if (game->cecp.ai_player == WHITE PLAYER)
    start_color = 1;
  else
    start_color = 0;

  /* Let's do castling check first */
  if ((castling = san_check_castling(san_str)) != 0)
  {
    snprintf(move, 5, "e%d%c%d", 8 * (1 - start_color),
	     (castling == 1) ? 'g' : 'c', 8 * (1 - start_color));

    return g_strdup(move);
  }

  if (game->cecp.debug)
    debug_output("DEBUG ( %11s line %3d ): Original san string %s",
		 __FILE__, __LINE__, san_str);

  start_piece = san_extract_piece(san_str);

  /* Remove the unneeded chars */
  san_simplify(san_str);

  if (game->cecp.debug)
    debug_output("DEBUG ( %11s line %3d ): Simplified san string %s\n",
		 __FILE__, __LINE__, san_str);

  /* Target information is already in the SAN string */
  target[0] = san_str[strlen(san_str) - 2] - 97;
  target[1] = san_str[strlen(san_str) - 1] - 49;

  if (game->cecp.debug)
    debug_output("DEBUG ( %11s line %3d ): Extracted target is %c%d\n",
		 __FILE__, __LINE__, target[0] + 'a', target[1] + 1);

  if (game->cecp.debug)
    debug_output
	("DEBUG ( %11s line %3d ): Extracted piece %d (internal format!)\n",
	 __FILE__, __LINE__, start_piece);

  if ((target_piece = game->board[target[0]][target[1]]) > 10)
  {
    target_color = 0;
    target_piece -= 10;
  } else if (target_piece != -1)
    target_color = 1;
  else
    target_color = -1;

  if (game->cecp.debug)
    debug_output
	("DEBUG ( %11s line %3d ): Target piece %d (internal format!)\n",
	 __FILE__, __LINE__, target_piece);

  /* Initialize start[] */
  start[0] = start[1] = -1;

  /* It's easy, if we have e.g. bd2 or 2e3 */
  if (strlen(san_str) == 3)
  {
    if (isalpha(san_str[0]))
      start[0] = san_str[0] - 97;
    if (isdigit(san_str[0]))
      start[1] = san_str[0] - 49;

    if (start[0] != -1)		/* If we got the start column */
    {
      if (game->cecp.debug)
	debug_output
	    ("DEBUG ( %11s line %3d ): San string contained column coordinate\n",
	     __FILE__, __LINE__);

      for (i = 0; i < 8; i++)
      {
	start[1] = i;

	if (game->board[start[0]][start[1]] ==
	    10 * (1 - start_color) + start_piece
	    && check_move(game, start, target, start_piece, start_color,
			  target_piece, target_color, 1) != -1)
	  break;
      }
    } else			/* If we got the start row */
    {
      if (game->cecp.debug)
	debug_output
	    ("DEBUG ( %11s line %3d ): San string contained row coordinate\n",
	     __FILE__, __LINE__);

      for (i = 0; i < 8; i++)
      {
	start[0] = i;

	if (game->board[start[0]][start[1]] ==
	    10 * (1 - start_color) + start_piece
	    && check_move(game, start, target, start_piece, start_color,
			  target_piece, target_color, 1) != -1)
	  break;
      }
    }

    snprintf(move, 5, "%c%d%c%d",
	     start[0] + 'a', start[1] + 1, target[0] + 'a', target[1] + 1);

    if (game->cecp.debug)
      debug_output("DEBUG ( %11s line %3d ): Converted san string to %s\n",
		   __FILE__, __LINE__, move);

    return g_strdup(move);
  } else
  {
    for (i = 0; i < 8; i++)
      for (j = 0; j < 8; j++)
	if (game->board[i][j] == 10 * (1 - start_color) + start_piece)
	{
	  start[0] = i;
	  start[1] = j;

	  if (check_move(game, start, target, start_piece, start_color,
			 target_piece, target_color, 1) != -1)
	  {
	    snprintf(move, 5, "%c%d%c%d",
		     start[0] + 'a', start[1] + 1,
		     target[0] + 'a', target[1] + 1);

	    if (game->cecp.debug)
	      debug_output
		  ("DEBUG ( %11s line %3d ): Converted san string to %s\n",
		   __FILE__, __LINE__, move);

	    return g_strdup(move);
	  }
	}
  }

  /* This should and hopefully will never happen ;-) */
  if (game->cecp.debug)
    debug_output("DEBUG ( %11s line %3d ): Couldn't convert move !!!\n"
		 "This is due to a strange bug in san.c. Please report this as soon as possible !!!\n",
		 __FILE__, __LINE__, move);
  gtk_main_quit();
  exit(EXIT_FAILURE);
}
